/*== R&S RSC Attenuator Controller ===========================================
 * LabWindows/CVI 2009 VXIpnp Instrument Driver
 * Original Release: September 2011
 * By: Petr Marcalik
 *
 * Should you have any technical questions please contact the hotline of
 * Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 * e-mail: CustomerSupport@rohde-schwarz.com
 *
 * Modification History:
 *
 *   see file ChangeLog
 *
 *===========================================================================*/
#ifdef _WIN32
  #ifdef _CVI_
    #include <userint.h>
  #else
    #include <windows.h>
  #endif
#endif
#include <math.h>
#include <visa.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>
#include "rsrsc.h"
#include "rsrsc_utility.h"
#include <limits.h>
#include <locale.h>  

/*= Macros ==================================================================*/

#define CHECKERR_CU(fCal) if (rsrsc_status = (fCal), rsrsc_status < VI_SUCCESS) \
    return rsrsc_initCleanUp (rmSession, instrSession, rsrsc_status); else

/*= DEFINES =================================================================*/
#define RSRSC_REVISION          "Rev 1.1.0, 02/2013, CVI 2009" /* Instrument driver revision */
#define BUFFER_SIZE             2048L    /* File I/O buffer size */

#define RSRSC_DEFAULT_TIMEOUT   10000   /* Default Timeout value in ms (10 sec) */

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/
static ViString correctionModeArr[] = {"REL", "ABS", VI_NULL};
static ViString userCalibrationArr[] = {"OFF","OFS","AF","FULL", VI_NULL};

/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
ViStatus rsrsc_initCleanUp (ViSession openRMSession, ViPSession openInstrSession,
                             ViStatus currentStatus);
ViStatus rsrsc_defaultInstrSetup (ViSession openInstrSession);

/*****************************************************************************
 * Function: Initialize
 * Purpose:  This function performs the following initialization actions:
 *           
 *           - Opens a session to the Default Resource Manager resource and a 
 *           session to the specified device using the interface and address 
 *           specified in the Resource_Name control.
 *           
 *           - Performs an identification query on the Instrument.
 *           
 *           - Resets the instrument to a known state.
 *           
 *           - Sends initialization commands to the instrument that set any 
 *           necessary programmatic variables such as Headers Off, Short 
 *           Command form, and Data Transfer Binary to the state necessary for 
 *           the operation of the instrument driver.
 *           
 *           - Returns an Instrument Handle which is used to differentiate 
 *           between different sessions of this instrument driver.
 *           
 *           - Each time this function is invoked a Unique Session is opened.  
 *           It is possible to have more than one session open for the same 
 *           resource.
 *           
 *****************************************************************************/
ViStatus _VI_FUNC rsrsc_Initialize(
    ViRsrc  resourceName,
    ViBoolean   idQuery,
    ViBoolean   resetDevice,
    ViSession*  instrSession
)
{
    ViStatus            rsrsc_status = VI_SUCCESS;
    ViSession           rmSession = 0;
    ViChar              buffer[BUFFER_SIZE] = "";
    rsrsc_instrRange    instrPtr = NULL;

    /* --- Set standard "C" locale --- */
    {
       char *locale;

       locale = setlocale(LC_ALL, "C");
    }
    
    /*- Open instrument session ---------------------------------------------*/
    CHECKERR( viOpenDefaultRM (&rmSession));

    if ((rsrsc_status = viOpen (rmSession, resourceName, VI_LOAD_CONFIG,
                                 VI_NULL, instrSession)) < 0)
    {
        (void)viClose (rmSession);
        return rsrsc_status;
    }

    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */

    if (instrPtr == NULL)
        instrPtr = calloc (1, sizeof (struct rsrsc_statusDataRanges));
        

    CHECKERR( viSetAttribute (*instrSession, VI_ATTR_USER_DATA, (ViAttrState)instrPtr));

    //With remote control via Ethernet, remote control is not automatically
    //set by means of a command. The instrument must be explicitly set to
    //the REMOTE state, e.g. by sending the intf command &GTR
    CHECKERR (viPrintf (*instrSession, "&GTR\n"));

    /*- Configure VISA Formatted I/O ----------------------------------------*/
    CHECKERR( viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE,
                                 RSRSC_DEFAULT_TIMEOUT));
    CHECKERR( viSetBuf (*instrSession, (ViUInt16) (VI_READ_BUF|VI_WRITE_BUF), 4000));
    CHECKERR( viSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE,
                                 VI_FLUSH_ON_ACCESS));
    CHECKERR( viSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE,
                                 VI_FLUSH_ON_ACCESS));

    /* -------------------------------------------------------------------- */
    /* Drawback of Agilent VISA is patched by adding extra termination      */
    /* character "\n" to each I/O operation                                 */
    /*                                                                      */
    /* VI_ATTR_SEND_END_EN will stay with this attribute set to VI_TRUE     */
    /* -------------------------------------------------------------------- */

    CHECKERR( viSetAttribute (*instrSession, VI_ATTR_SEND_END_EN, VI_TRUE));

    // read/reset ESR which may still indicate device dependent error from 
    // previous handling!
    (void) rsrsc_checkStatus(*instrSession);
    
    /*- Identification Query ------------------------------------------------*/
   
     if (idQuery != VI_FALSE)
     {
         CHECKERR( viQueryf (*instrSession, "*IDN?\n","%*[^,],%[^,]",
                                buffer));

         /* Check if instrument is supported or not */
         if (strstr(buffer, "RSC") == NULL)
             return rsrsc_initCleanUp (rmSession, instrSession, VI_ERROR_FAIL_ID_QUERY);
     }

        /* Check if instrument is supported or not */
     
    /*- Reset instrument ----------------------------------------------------*/
    if (resetDevice != VI_FALSE)
        CHECKERR( rsrsc_reset (*instrSession));
    else  /*- Send Default Instrument Setup ---------------------------------*/
        CHECKERR( rsrsc_defaultInstrSetup (*instrSession));

    return rsrsc_status;
}


/*****************************************************************************
 * Function: Configure Attenuation 
 * Purpose:  This function sets the attenuation for the specified attenuator.
 *****************************************************************************/
ViStatus _VI_FUNC rsrsc_ConfigureAttenuation (ViSession instrSession,
                                              ViInt32 attenuator,
                                              ViReal64 attenuation)
{
    return rsrsc_setViReal64Atten (instrSession,
                                      "ATT%ld:ATT",
									  attenuator,
                                      attenuation,
									  0.0,0.0,
                                      VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Configure Attenuator Cascading 
 * Purpose:  This function sets the cascading of external attenuators.
 *****************************************************************************/
ViStatus _VI_FUNC rsrsc_ConfigureAttenuatorCascading (ViSession instrSession,
                                                      ViInt32 attenuator,
                                                      ViBoolean status,
													  ViString attenuators)
{
	ViChar buffer[BUFFER_SIZE] = "";
    
    if (rsrsc_invalidViInt32Range (attenuator, 6, 7) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "ATT%ld:CASC? %ld,%s", attenuator, status, attenuators);
    
	return rsrsc_sendCmd (instrSession, buffer);    
}

/*****************************************************************************
 * Function: Configure Correction Mode
 * Purpose:  This function turns the correction mode for the specified
 *           attenuator on or off.
 *****************************************************************************/
ViStatus _VI_FUNC rsrsc_ConfigureCorrectionMode (ViSession instrSession,
                                                 ViInt32 attenuator,
                                                 ViBoolean correctionMode)
{
    return rsrsc_setViBooleanAtten (instrSession, 
                               "ATT%ld:CORR",
							   attenuator,
                               correctionMode, 
                               VI_ERROR_PARAMETER3);
}

/*****************************************************************************
 * Function: Configure Frequency
 * Purpose:  This function sets the frequency for which the attenuation
 *           deviation should be calculated.
 *****************************************************************************/
ViStatus _VI_FUNC rsrsc_ConfigureFrequency (ViSession instrSession,
                                            ViInt32 attenuator, ViReal64 frequency)
{
   
    return rsrsc_setViReal64Atten (instrSession,
                                      "ATT%ld:FREQ",
									  attenuator,
                                      frequency,
									  0.0,0.0,
                                      VI_ERROR_PARAMETER3);
}
/*****************************************************************************
 * Function: Configure User Calibration
 * Purpose:  This function configures internal attenuator's calibration.
 *****************************************************************************/
ViStatus _VI_FUNC rsrsc_ConfigureUserCalibration (ViSession instrSession,
                                                  ViInt32 attenuator,
                                                  ViInt32 correctionMode,
                                                  ViInt32 userCalibration,
												  ViReal64 offset)
{
    ViStatus rsrsc_status = VI_SUCCESS;
  
	rsrsc_status = rsrsc_setEnumAtten (instrSession,
				                        "ATT%ld:CMOD",
				                        attenuator,
				                        correctionMode,
				                        correctionModeArr,
				                        VI_ERROR_PARAMETER3);
	
	rsrsc_status = rsrsc_setEnumAtten (instrSession,
				                        "ATT%ld:UCAL",
				                        attenuator,
				                        userCalibration,
				                        userCalibrationArr,
				                        VI_ERROR_PARAMETER4);	


    rsrsc_status = rsrsc_setViReal64Atten (instrSession,
                                      "ATT%ld:UCAL:OFFS",
									  attenuator,
                                      offset,
									  -100.0,100.0,
                                      VI_ERROR_PARAMETER5);

	return rsrsc_status;
}

/*****************************************************************************
 * Function: Configure System Date 
 * Purpose:  This function sets the date for device's internal calendar.
 *****************************************************************************/              
ViStatus _VI_FUNC rsrsc_ConfigureSystemDate (ViSession instrSession,
                                             ViInt32 year, ViInt32 month, ViInt32 day)
{
	ViChar buffer[BUFFER_SIZE] = "";
    
    if (rsrsc_invalidViInt32Range (year, 1980, 2099) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	if (rsrsc_invalidViInt32Range (month, 1, 12) == VI_TRUE)
        return VI_ERROR_PARAMETER3;
	if (rsrsc_invalidViInt32Range (day, 1, 31) == VI_TRUE)
        return VI_ERROR_PARAMETER4;
	
	sprintf (buffer, "SYST:DATE %ld,%ld,%ld", year, month, day);
    
	return rsrsc_sendCmd (instrSession, buffer);    
}

/*****************************************************************************
 * Function: Configure System Time
 * Purpose:  This function sets the internal clock.
 *****************************************************************************/                                          
ViStatus _VI_FUNC rsrsc_ConfigureSystemTime (ViSession instrSession,
                                             ViInt32 hour, ViInt32 minute,
                                             ViInt32 second)
{
	ViChar buffer[BUFFER_SIZE] = "";
    
    if (rsrsc_invalidViInt32Range (hour, 0, 23) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	if (rsrsc_invalidViInt32Range (minute, 0, 59) == VI_TRUE)
        return VI_ERROR_PARAMETER3;
	if (rsrsc_invalidViInt32Range (second, 0, 59) == VI_TRUE)
        return VI_ERROR_PARAMETER4;
	
	sprintf (buffer, "SYST:TIME %ld,%ld,%ld", hour, minute, second);
    
	return rsrsc_sendCmd (instrSession, buffer);    
}

/*****************************************************************************
 * Function: Query Attenuator Deviation
 * Purpose:  This function returns the attenuation deviation for the
 *			 specified attenuator at the set frequency.
 *****************************************************************************/
ViStatus _VI_FUNC rsrsc_QueryAttenuatorDeviation (ViSession instrSession,
                                                  ViInt32 attenuator,
                                                  ViReal64 *deviation)
{
    ViStatus rsrsc_status = VI_SUCCESS;
    
    rsrsc_status = rsrsc_getViReal64Atten (instrSession,
                                      "ATT%ld:DEV?",
									  attenuator,
                                      deviation,
                                      VI_ERROR_PARAMETER3);
    return rsrsc_status;
}

/*****************************************************************************
 * Function: Query Attenuator List
 * Purpose:  This function returns the attenuators that are available.
 *****************************************************************************/
ViStatus _VI_FUNC rsrsc_QueryAttenuatorList (ViSession instrSession,
											 ViInt32 bufferSize,
                                             ViChar attenuatorList[])
{
    ViStatus rsrsc_status = VI_SUCCESS;
    
    rsrsc_status = rsrsc_getViString (instrSession,
                            "ATT:ENUM?",
                            attenuatorList,
                            bufferSize,
                            VI_ERROR_PARAMETER2);
	return rsrsc_status;
}

/*****************************************************************************
 * Function: Query Attenuator Name
 * Purpose:  This function returns the attenuator's name.
 *****************************************************************************/
ViStatus _VI_FUNC rsrsc_QueryAttenuatorName (ViSession instrSession,
                                             ViInt32 attenuator,
											 ViInt32 bufferSize,
                                             ViChar attenuatorName[])
{
    ViStatus rsrsc_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE] = "";
    
    if (rsrsc_invalidViInt32Range (attenuator, 2, 5) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "ATT%ld:NAME?", attenuator);
	
    rsrsc_status = rsrsc_getViString (instrSession,
                            buffer,
                            attenuatorName,
                            bufferSize,
                            VI_ERROR_PARAMETER3);
	return rsrsc_status;
}

/*****************************************************************************
 * Function: Query Attenuator Catalog List
 * Purpose:  This function queries a string list from all attenuators.
 *****************************************************************************/
ViStatus _VI_FUNC rsrsc_QueryAttenuatorCatalogList (ViSession instrSession,
                                                    ViInt32 bufferSize,
                                             		ViChar attenuatorCatalogList[])
{
    ViStatus rsrsc_status = VI_SUCCESS;
	
    rsrsc_status = rsrsc_getViString (instrSession,
                            		  "ATT:CAT?",
                            		  attenuatorCatalogList,
                                      bufferSize,
                                      VI_ERROR_PARAMETER3);
	return rsrsc_status;
}

/*****************************************************************************
 * Function: Load Correction Data File
 * Purpose: This function loads a file with correction data for the
specified attenuator.

Remote-control command(s):
ATTenuator<n>:UCALibration:LOAD <File>

 *****************************************************************************/
ViStatus _VI_FUNC rsrsc_LoadCorrectionDataFile (ViSession instrSession,
	                                             ViInt32 attenuator,
                                                 ViString fileName)
{
    ViStatus rsrsc_status = VI_SUCCESS;
	ViChar buffer[BUFFER_SIZE] = "";
    
    if (rsrsc_invalidViInt32Range (attenuator, 1, 7) == VI_TRUE)
        return VI_ERROR_PARAMETER2;
	
	sprintf (buffer, "ATT%ld:UCAL:LOAD", attenuator);
	
    rsrsc_status = rsrsc_setViStringQuoted (instrSession,
                            buffer,
                            fileName,
                            "'",
                            VI_ERROR_PARAMETER3);
	return rsrsc_status;
}

/*****************************************************************************
 * Function: Query Hardware Information
 * Purpose:  This function returns the hardware information.
 *****************************************************************************/
ViStatus _VI_FUNC rsrsc_QueryHardwareInformation (ViSession instrSession,
												  ViInt32 bufferSize,
                                                  ViChar hardwareInformation[])
{
    ViStatus rsrsc_status = VI_SUCCESS;
    
    rsrsc_status = rsrsc_getViString (instrSession,
                            "DIAG:SERV:HWIN?",
                            hardwareInformation,
                            bufferSize,
                            VI_ERROR_PARAMETER2);
	return rsrsc_status;
}

/*****************************************************************************
 * Function: Query Software Information
 * Purpose:  This function returns the software information.
 *****************************************************************************/
ViStatus _VI_FUNC rsrsc_QuerySoftwareInformation (ViSession instrSession,
											  	  ViInt32 bufferSize,		
											      ViChar softwareInformation[])
{
    ViStatus rsrsc_status = VI_SUCCESS;
    
    rsrsc_status = rsrsc_getViString (instrSession,
                            "DIAG:SERV:SWIN?",
                            softwareInformation,
                            bufferSize,
                            VI_ERROR_PARAMETER2);
	return rsrsc_status;
}
 

/*****************************************************************************
 * Function: State Checking
 * Purpose:  This function switches state checking of the instrument (reading 
 *           of the Standard Event Register and checking it for error) status 
 *           subsystem. Driver functions are using state checking which is by 
 *           default enabled.
 *           
 *           Note:
 *           
 *           (1) In debug mode enable state checking.
 *           
 *           (2) For better bus throughput and instruments performance disable 
 *           state checking.
 *           
 *           (3) When state checking is disabled driver does not check if 
 *           correct instrument model or option is used with each of the 
 *           functions. This might cause unexpected behaviour of the 
 *           instrument.
 *****************************************************************************/
ViStatus _VI_FUNC rsrsc_errorCheckState(
    ViSession   instrSession,
    ViBoolean   stateChecking
)
{
    ViStatus    rsrsc_status = VI_SUCCESS;
    rsrsc_instrRange instrPtr = NULL;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    CHECKNULL (instrPtr, 1);
    
    instrPtr -> errorChecking = stateChecking;

    return rsrsc_status;
}

/*****************************************************************************
 * Function: Write To Instrument
 * Purpose:  This function writes commands and queries to the instrument to 
 *           modify parameters and query device settings.
 *****************************************************************************/
ViStatus _VI_FUNC rsrsc_writeInstrData(
    ViSession   instrSession,
    ViString    writeBuffer
)
{
    ViStatus rsrsc_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, "%s\n", writeBuffer));

    return rsrsc_status;
}

/*****************************************************************************
 * Function: Read Instrument Data
 * Purpose:  This function reads data from the instrument's output buffer and 
 *           returns it to the specified variable in memory.
 *           
 *           Notes:
 *           
 *           (1) Because the instrument may return both numeric and text data 
 *           in response to queries this function returns the data in string 
 *           format.
 *           
 *           (2) If valid data is not available at the instrument's output 
 *           buffer when this function is called the instrument will hang up 
 *           and the function will not return until it times out.  If the 
 *           time-out is disabled this function will hang indefinitely and it 
 *           may be necessary to reboot the computer to break out.
 *****************************************************************************/
ViStatus _VI_FUNC rsrsc_readInstrData(
    ViSession   instrSession,
    ViInt32 numberBytesToRead,
    ViChar _VI_FAR readBuffer[],
    ViInt32*    numbytesRead
)
{
    ViStatus    rsrsc_status = VI_SUCCESS;
    ViUInt32    tmpCnt = 0;

    if (numbytesRead != NULL)
        *numbytesRead = 0L;

    CHECKERR (viRead (instrSession, (ViBuf)readBuffer, (ViUInt32) numberBytesToRead, &tmpCnt));

    if (numbytesRead)
        *numbytesRead = (ViInt32)tmpCnt;

    return rsrsc_status;
}

/*****************************************************************************
 * Function: Reset
 * Purpose:  RESET resets the instrument to a defined default state. The 
 *           function has the same effect as pressing the PRESET key.
 *****************************************************************************/
ViStatus _VI_FUNC rsrsc_reset(
    ViSession   instrSession
)
{
    ViStatus rsrsc_status = VI_SUCCESS;

    /*  Initialize the instrument to a known state.  */
    CHECKERR (viPrintf (instrSession, "*RST;*CLS\n"));
    CHECKERR (rsrsc_defaultInstrSetup (instrSession));

    return rsrsc_status;
}

/*****************************************************************************
 * Function: Self-Test
 * Purpose:  This function runs the instrument's self test routine and returns 
 *           the test result(s).
 *****************************************************************************/
ViStatus _VI_FUNC rsrsc_SelfTest (
    ViSession   instrSession,
    ViInt16*    selfTestResult,
    ViChar _VI_FAR selfTestMessage[]
)
{
    ViStatus rsrsc_status = VI_SUCCESS;

    CHECKERR( viQueryf (instrSession, "*TST?\n", "%hd", selfTestResult)); 
	CHECKERR( viQueryf (instrSession, "DIAG:SERV:STES:RES?\n", "%s", selfTestMessage)); 

   // sprintf (selfTestMessage, "%s", (*selfTestResult == 0) ? "Self-Test Passed" : "Self-Test Failed");

    return rsrsc_status;
}

/*****************************************************************************
 * Function: Error-Query
 * Purpose:  This function reads an error code from the instrument's error 
 *           queue.
 *****************************************************************************/
ViStatus _VI_FUNC rsrsc_ErrorQuery(
    ViSession   instrSession,
    ViInt32*    errorCode,
    ViChar _VI_FAR errorMessage[]
)
{
    ViStatus rsrsc_status = VI_SUCCESS;
    ViInt32  code = 0;
    ViChar   msg [256] = "";

    CHECKERR (viPrintf (instrSession, ":SYST:ERR?\n"));
    CHECKERR (viScanf (instrSession, "%ld,\"%[^\"]", &code, msg));
    if (errorCode != NULL)
        *errorCode = code;
    
    if (errorMessage != NULL)
        strncpy (errorMessage, msg, 255);

    return rsrsc_status;
}

/*****************************************************************************
 * Function: Error Message
 * Purpose:  This function takes the Status Code returned by the instrument 
 *           driver functions, interprets it and returns it as a user readable 
 *           string.
 *****************************************************************************/
ViStatus _VI_FUNC rsrsc_ErrorMessage(
    ViSession   instrSession,
    ViStatus    statusCode,
    ViChar _VI_FAR errMessage[]
)
{
    ViStatus rsrsc_status = VI_SUCCESS;
    ViInt16 i;

    static rsrsc_tStringValPair statusDescArray[] = {
        {VI_WARN_NSUP_ID_QUERY,                 "WARNING: ID Query not supported"},
        {VI_WARN_NSUP_RESET,                    "WARNING: Reset not supported"},
        {VI_WARN_NSUP_SELF_TEST,                "WARNING: Self-test not supported"},
        {VI_WARN_NSUP_ERROR_QUERY,              "WARNING: Error Query not supported"},
        {VI_WARN_NSUP_REV_QUERY,                "WARNING: Revision Query not supported"},
        {VI_ERROR_PARAMETER1,                   "ERROR: Parameter 1 out of range"},
        {VI_ERROR_PARAMETER2,                   "ERROR: Parameter 2 out of range"},
        {VI_ERROR_PARAMETER3,                   "ERROR: Parameter 3 out of range"},
        {VI_ERROR_PARAMETER4,                   "ERROR: Parameter 4 out of range"},
        {VI_ERROR_PARAMETER5,                   "ERROR: Parameter 5 out of range"},
        {VI_ERROR_PARAMETER6,                   "ERROR: Parameter 6 out of range"},
        {VI_ERROR_PARAMETER7,                   "ERROR: Parameter 7 out of range"},
        {VI_ERROR_PARAMETER8,                   "ERROR: Parameter 8 out of range"},
        {VI_ERROR_FAIL_ID_QUERY,                "ERROR: Identification query failed"},
        {VI_ERROR_INV_RESPONSE,                 "ERROR: Interpreting instrument response"},
        {VI_ERROR_INSTR_FILE_OPEN,              "ERROR: Opening the specified file"},
        {VI_ERROR_INSTR_FILE_WRITE,             "ERROR: Writing to the specified file"},
        {VI_ERROR_INSTR_INTERPRETING_RESPONSE,  "ERROR: Interpreting the instrument's response"},
        {VI_ERROR_INSTR_PARAMETER8,             "ERROR: Parameter 8 out of range"},
        {VI_ERROR_INSTR_PARAMETER9,             "ERROR: Parameter 9 out of range"},
        {VI_ERROR_INSTR_PARAMETER10,            "ERROR: Parameter 10 out of range"},
        {VI_ERROR_INSTR_PARAMETER11,            "ERROR: Parameter 11 out of range"},
        {VI_ERROR_INSTR_PARAMETER12,            "ERROR: Parameter 12 out of range"},
        {VI_ERROR_INSTR_PARAMETER13,            "ERROR: Parameter 13 out of range"},
        {VI_ERROR_INSTR_PARAMETER14,            "ERROR: Parameter 14 out of range"},
        {VI_ERROR_INSTR_PARAMETER15,            "ERROR: Parameter 15 out of range"},

        /* --- Instrument specific error codes --- */

        {RSRSC_ERROR_INSTRUMENT_ERROR,          "ERROR: Instrument status error"},
        {RSRSC_ERROR_INVALID_CONFIGURATION,     "ERROR: Instrument configuration error"},
        {RSRSC_ERROR_INSTRUMENT_OPTION,         "ERROR: Option not installed"},
        {RSRSC_ERROR_INSTRUMENT_NSUP_MODEL,     "ERROR: Required Instrument model is not available."},
        {RSRSC_ERROR_SETTINGS_CONFLICT,         "ERROR: Instrument settings conflict."},
        {RSRSC_ERROR_CALIBRATION_FAILED,        "ERROR: Calibration failed."},

        /* --- Instrument specific warnings --- */

        {RSRSC_WARNING_SMALL_BUFFER,            "WARNING: Buffer too small"},

        {VI_NULL, VI_NULL}
    };

    rsrsc_status = viStatusDesc (instrSession, statusCode, errMessage);
    if (rsrsc_status == VI_WARN_UNKNOWN_STATUS)
        {
        for (i = 0; statusDescArray[i].stringName; i++)
            {
            if (statusDescArray[i].stringVal == statusCode)
                {
                sprintf (errMessage, "%s", statusDescArray[i].stringName);
                return (VI_SUCCESS);
                }
            }
        sprintf (errMessage, "Unknown Error 0x%X", (unsigned int) statusCode);
        return (VI_WARN_UNKNOWN_STATUS);
        }

    rsrsc_status = VI_SUCCESS;
    return rsrsc_status;
}

/*****************************************************************************
 * Function: Revision Query
 * Purpose:  This function returns the revision numbers of the instrument 
 *           driver and instrument firmware, and tells the user with which  
 *           instrument firmware this revision of the driver is compatible.
 *****************************************************************************/
ViStatus _VI_FUNC rsrsc_RevisionQuery(
    ViSession   instrSession,
    ViChar _VI_FAR instrumentDriverRevision[],
    ViChar _VI_FAR firmwareRevision[]
)
{
    ViStatus rsrsc_status = VI_SUCCESS;

    CHECKERR (viPrintf (instrSession, "*IDN?\n"));
    CHECKERR (viScanf (instrSession, "%*[^,],%*[^,],%*[^,],%[^\r\n]", firmwareRevision));
  //	CHECKERR (viScanf (instrSession, "%*[^-]-%[^ (\r\n]", firmwareRevision));

    sprintf (instrumentDriverRevision, "%s", RSRSC_REVISION);

    return rsrsc_status;
}

/*****************************************************************************
 * Function: Read All System Messages 
 * Purpose:  This function reads out all system messages.
 *****************************************************************************/
ViStatus _VI_FUNC rsrsc_ReadAllSystemMessages (ViSession instrSession,
											   ViInt32 bufferSize,
                                               ViChar systemMessages[])
{
    ViStatus rsrsc_status = VI_SUCCESS;
    
    rsrsc_status = rsrsc_getViString (instrSession,
                            "SYST:ERR:LIST?",
                            systemMessages,
                            bufferSize,
                            VI_ERROR_PARAMETER2);
	return rsrsc_status;
}

/*****************************************************************************
 * Function: Clear All System Messages
 * Purpose:  This function deletes all entries in the SYSTEM MESSAGES table.
 *****************************************************************************/
ViStatus _VI_FUNC rsrsc_ClearAllSystemMessages (ViSession instrSession)
{

	return rsrsc_sendCmd (instrSession, "SYST:ERR:CLE:ALL");    
}

	

/*****************************************************************************
 * Function: Close
 * Purpose:  This function performs the following operations:
 *           viClose (instrSession) and viClose (rmSession).
 *           
 *           Notes:
 *           
 *           (1) The instrument must be reinitialized to use it again.
 *****************************************************************************/
ViStatus _VI_FUNC rsrsc_Close(
    ViSession   instrSession
)
{
    rsrsc_instrRange instrPtr = NULL;
    ViSession rmSession = 0;
    ViStatus rsrsc_status = VI_SUCCESS;

	
    CHECKERR( viGetAttribute (instrSession, VI_ATTR_RM_SESSION, &rmSession));
    CHECKERR( viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
	viPrintf (instrSession, "&GTL");
	 
    if (instrPtr != NULL)
        free (instrPtr);

    rsrsc_status = viClose (instrSession);
    (void) viClose (rmSession);

    return rsrsc_status;
}
		  
/*===========================================================================*/
/* Function: Default Instrument Setup                                        */
/* Purpose:  This function sends a default setup to the instrument.  This    */
/*           function is called by the rsrsc_reset operation and by the     */
/*           rsrsc_init function if the reset option has not been           */
/*           selected.  This function is useful for configuring any          */
/*           instrument settings that are required by the rest of the        */
/*           instrument driver functions such as turning headers ON or OFF   */
/*           or using the long or short form for commands, queries, and data.*/
/*===========================================================================*/
ViStatus rsrsc_defaultInstrSetup (ViSession instrSession)
{
    ViChar              buffer[BUFFER_SIZE] = "";

    ViStatus            rsrsc_status = VI_SUCCESS;
    rsrsc_instrRange    instrPtr = NULL;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));
    
    CHECKNULL (instrPtr, 1);
    
    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */
    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    instrPtr -> errorChecking = VI_TRUE;
    strcpy (instrPtr -> instrDriverRevision, RSRSC_REVISION);

    /* Check for instrument version */
    CHECKERR (viQueryf (instrSession, "*IDN?\n", "%*[^,],%[^,],%*[\r\n]", buffer));
    
    return rsrsc_status;
}

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the rsrsc_init function.  When   */
/*           an error is detected this function is called to close the       */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from rsrsc_init to       */
/*           VI_NULL.                                                        */
/*===========================================================================*/
ViStatus rsrsc_initCleanUp (ViSession openRMSession,
                             ViPSession openInstrSession,
                             ViStatus currentStatus)
{
    rsrsc_instrRange instrPtr = NULL;

    if (viGetAttribute (*openInstrSession, VI_ATTR_USER_DATA, &instrPtr) >= 0)
        if (instrPtr != NULL)
            free (instrPtr);

    (void) viClose (*openInstrSession);
    (void) viClose (openRMSession);
    *openInstrSession = VI_NULL;

    return currentStatus;
}

