/*===========================================================================*/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*===========================================================================*/
#ifdef _WIN32
  #ifdef _CVI_
    #include <userint.h>
  #else
    #include <windows.h>
  #endif
#endif
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "rsrsc.h"
#include "rsrsc_utility.h"

#define IEEE_ERROR_BITS         0x3C    /* AND mask for ESR checking */
#define BUFFER_SIZE             2048UL    /* File I/O buffer size */

/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean rsrsc_invalidViBooleanRange (ViBoolean val)
{
    return(ViBoolean)((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean rsrsc_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Unsigned Integer Value Out Of Range - ViUInt32             */
/* Purpose:  This function checks a long signed integer value to see if it   */
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean rsrsc_invalidViUInt32Range (ViUInt32 val, ViUInt32 min, ViUInt32 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean rsrsc_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}
/*=========================================================================*/
/* Function: Check Status                                                  */
/* Purpose:  This function reads Event Status Register to check for        */
/*           possible errors.                                              */
/*=========================================================================*/
ViStatus rsrsc_checkStatus (ViSession instrSession)
{
    ViStatus            rsrsc_status = VI_SUCCESS;
    rsrsc_instrRange    instrPtr = NULL;

    CHECKERR (viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    /* Error Checking Enabled? */
    if (instrPtr != NULL)
    {
        if (instrPtr -> errorChecking != VI_FALSE)
        {
            ViInt32 esrByte = 0;

            /*--------------- Read the Standard Event Register ------------------*/
            CHECKERR (viQueryf (instrSession, "*ESR?\n", "%ld", &esrByte));

            /*---------------- Check if any error or message bit was asserted ---*/
            rsrsc_status = ((esrByte & IEEE_ERROR_BITS) != 0) ? RSRSC_ERROR_INSTRUMENT_ERROR : VI_SUCCESS;
        }
    }

    return rsrsc_status;
}

/*===========================================================================*/
/* Function: Send Command                                                    */
/* Purpose:  This function sends command to the instrument with error check. */
/*===========================================================================*/
ViStatus rsrsc_sendCmd (ViSession instrSession, ViString cmd)
{
    ViStatus    rsrsc_status    = VI_SUCCESS;
    size_t      length          = 0;
    ViChar      *pbuffer;

    length = strlen (cmd);
    pbuffer = (ViChar*) malloc(length + 1);
    if (pbuffer == NULL)
        return VI_ERROR_ALLOC;

    memcpy (pbuffer, cmd, length);
    pbuffer[length] = '\n';

    length++;
    GOCHECKERR (viWrite (instrSession, (ViPBuf) cmd, (ViUInt32) length, VI_NULL));
    GOCHECKERR (rsrsc_checkStatus (instrSession));

ERROR_LABEL:

    free(pbuffer);

    return rsrsc_status;
}

/*===========================================================================*/
/* Function: Set ViInt32                                                     */
/* Purpose:  This function sets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsrsc_setViInt32 (ViSession instrSession,
                           ViString cmd,
                           ViInt32 value,
                           ViInt32 min,
                           ViInt32 max,
                           ViStatus parameterErrorCode)
{
    ViChar      *wrt_buf;
    ViStatus    rsrsc_status = VI_SUCCESS;

    if ((max - min) > 0)
        if (rsrsc_invalidViInt32Range (value, min, max) == VI_TRUE)
            return parameterErrorCode;

    
    wrt_buf = (ViChar*) malloc (strlen (cmd) + 20);
    
    sprintf (wrt_buf, "%s %ld", cmd, value);

    rsrsc_status = rsrsc_sendCmd (instrSession, wrt_buf);
    
    free (wrt_buf);
    return rsrsc_status;
}

/*===========================================================================*/
/* Function: Get ViInt32                                                     */
/* Purpose:  This function gets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsrsc_getViInt32 (ViSession instrSession,
                           ViString cmd,
                           ViInt32 *value,
                           ViStatus parameterErrorCode)
{
    ViStatus    rsrsc_status = VI_SUCCESS;

    if (value == VI_NULL)
        return parameterErrorCode;

    CHECKERR( viPrintf(instrSession, "%s\n", cmd));
    CHECKERR( viScanf(instrSession, "%ld", value));
    CHECKERR( rsrsc_checkStatus (instrSession));

    return rsrsc_status;
}

/*===========================================================================*/
/* Function: Set ViBoolean                                                   */
/* Purpose:  This function sets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsrsc_setViBoolean (ViSession instrSession,
                           ViString cmd,
                           ViBoolean value,
                           ViStatus parameterErrorCode)
{
    if (rsrsc_invalidViBooleanRange (value) == VI_TRUE)
        return parameterErrorCode;

    return rsrsc_setViString (instrSession,
                              cmd,
                              (value == VI_TRUE) ? "ON" : "OFF",
                              0);
}

/*===========================================================================*/
/* Function: Get ViBoolean                                                   */
/* Purpose:  This function gets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsrsc_getViBoolean (ViSession instrSession,
                           ViString cmd,
                           ViBoolean *value,
                           ViStatus parameterErrorCode)
{
    ViStatus    rsrsc_status;
    ViChar      buffer[4];

    if (value == NULL)
        return parameterErrorCode;

    CHECKERR (rsrsc_getViString (instrSession,
                                cmd,
                                buffer, 4,
                                0));

    *value = (ViBoolean) ((strncmp (buffer, "1", 1) == 0) || (strncmp (buffer,"ON",2) == 0));

    return rsrsc_status;
}

/*===========================================================================*/
/* Function: Set ViBoolean With Attenuator                                   */
/* Purpose:  This function sets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsrsc_setViBooleanAtten (ViSession instrSession,
                           ViString cmd,
                           ViInt32 attenuator,
                           ViBoolean value,
                           ViStatus parameterErrorCode)
{
    ViChar  wrt_buf[BUFFER_SIZE];
    ViChar* p2buf;

    CHECKATTENUATOR;
    if (rsrsc_invalidViBooleanRange (value) != 0)
        return parameterErrorCode;

    p2buf = wrt_buf + sprintf(wrt_buf,cmd, attenuator);
    sprintf (p2buf, " %s", (value != 0) ? "ON" : "OFF");

    return rsrsc_sendCmd (instrSession, wrt_buf);
}

/*===========================================================================*/
/* Function: Get ViBoolean With Attenuator                                   */
/* Purpose:  This function gets command with integer parameter.              */
/*===========================================================================*/
ViStatus rsrsc_getViBooleanAtten (ViSession instrSession,
                           ViString cmd,
                           ViInt32 attenuator,
                           ViBoolean *value,
                           ViStatus parameterErrorCode)
{
    ViChar  wrt_buf[BUFFER_SIZE];

    CHECKATTENUATOR;

    sprintf(wrt_buf,cmd, attenuator);

    return rsrsc_getViBoolean(instrSession,
                                   wrt_buf,
                                   value,
                                   parameterErrorCode);
}

/*===========================================================================*/
/* Function: Set ViReal64                                                    */
/* Purpose:  This function sets command with double parameter.               */
/*===========================================================================*/
ViStatus rsrsc_setViReal64 (ViSession instrSession,
                            ViString cmd,
                            ViReal64 value,
                            ViReal64 min,
                            ViReal64 max,
                            ViStatus parameterErrorCode)
{
    ViChar      buffer[BUFFER_SIZE];

    /* do range checking only if there's difference between max and min */
    if ((max - min) > 1.0e-14)
        if (rsrsc_invalidViReal64Range (value, min, max) == VI_TRUE)
            return parameterErrorCode;

    sprintf(buffer,"%s %E", cmd, value);

    return rsrsc_sendCmd(instrSession, buffer);
}

/*===========================================================================*/
/* Function: Get ViReal64                                                    */
/* Purpose:  This function gets command with double parameter.               */
/*===========================================================================*/
ViStatus rsrsc_getViReal64 (ViSession instrSession,
                            ViString cmd,
                            ViReal64 *value,
                            ViStatus parameterErrorCode)
{
    ViStatus    rsrsc_status = VI_SUCCESS;

    if ( value == VI_NULL )
        return parameterErrorCode;

    CHECKERR(viPrintf(instrSession, "%s\n",cmd));
    CHECKERR(viScanf (instrSession, "%le",value));
    CHECKERR(rsrsc_checkStatus (instrSession));

    return rsrsc_status;
}

/*===========================================================================*/
/* Function: Set ViReal64 With Attenuator                                    */
/* Purpose:  This function sets command with double parameter.               */
/*===========================================================================*/
ViStatus rsrsc_setViReal64Atten (ViSession instrSession,
                            ViString cmd,
                            ViInt32  attenuator,
                            ViReal64 value,
                            ViReal64 min,
                            ViReal64 max,
                            ViStatus parameterErrorCode)
{
    ViChar      wrt_buf[BUFFER_SIZE];

    CHECKATTENUATOR;

    sprintf(wrt_buf,cmd, attenuator);

    return rsrsc_setViReal64(instrSession,
                             wrt_buf,
                             value,
                             min,
                             max,
                             parameterErrorCode);
}

/*===========================================================================*/
/* Function: Get ViReal64 With Attenuator                                    */
/* Purpose:  This function gets command with double parameter.               */
/*===========================================================================*/
ViStatus rsrsc_getViReal64Atten (ViSession instrSession,
                            ViString cmd,
                            ViInt32 attenuator,
                            ViReal64 *value,
                            ViStatus parameterErrorCode)
{
    ViChar      wrt_buf[BUFFER_SIZE];

    CHECKATTENUATOR;

    sprintf(wrt_buf, cmd, attenuator);

    return rsrsc_getViReal64(instrSession,
                             wrt_buf,
                             value,
                             parameterErrorCode);
}


/*===========================================================================*/
/* Function: Set String                                                      */
/* Purpose:  This function sets command with string parameter.               */
/*===========================================================================*/
ViStatus rsrsc_setViString (ViSession instrSession,
                            ViString cmd,
                            ViString value,
                            ViStatus parameterErrorValue)
{
    ViStatus    rsrsc_status = VI_SUCCESS;
    ViChar*     wrt_buf;
    ViChar*     p2buf;

    if (value == VI_NULL)
        return parameterErrorValue;

    wrt_buf = (ViChar*) malloc (strlen(cmd) + strlen(value) + 4 );
    p2buf = wrt_buf;
    p2buf += sprintf(wrt_buf, "%s %s\n", cmd, value);

    GOCHECKERR(viWrite (instrSession, (ViBuf)wrt_buf, (ViUInt32) (p2buf - wrt_buf), VI_NULL));
    GOCHECKERR(rsrsc_checkStatus(instrSession));

ERROR_LABEL:

    free(wrt_buf);

    return rsrsc_status;
}

/*===========================================================================*/
/* Function: Get String                                                      */
/* Purpose:  This function gets command with string parameter.               */
/*===========================================================================*/
ViStatus rsrsc_getViString (ViSession instrSession,
                            ViString cmd,
                            ViString value,
                            ViUInt32 buffer_size,
                            ViStatus parameterErrorCode)
{
    ViStatus    rsrsc_status    = VI_SUCCESS;
    ViChar      *pbuffer,
                *p2buffer;
    ViUInt32    count           = 0;
    size_t      length          = 0;

    if (value == NULL)
        return parameterErrorCode;

    length = strlen (cmd);
    pbuffer = (ViChar*) malloc (length + 1);
    
    memcpy (pbuffer, cmd, length);
    if (pbuffer == NULL)
        return VI_ERROR_ALLOC;
    
    pbuffer[length] = '\n';

    GOCHECKERR (viWrite (instrSession, (ViPBuf)pbuffer, (ViUInt32) (length + 1), VI_NULL));

    free (pbuffer);
    
    pbuffer = NULL;

    GOCHECKERR (rsrsc_readDataUnknownLength (instrSession, &pbuffer, &count));
    
    if (pbuffer == NULL)
        return rsrsc_status;
    
    GOCHECKERR (rsrsc_checkStatus (instrSession));

    if (count > buffer_size)
    {
        rsrsc_status = RSRSC_WARNING_SMALL_BUFFER;
        count = buffer_size;
        memcpy (value, pbuffer, (size_t) count);
        value[count] = '\0';
    }
    else
        if ((p2buffer = strpbrk (pbuffer, "\r\n")) != VI_NULL)
        {
            rsrsc_status = VI_SUCCESS;
            memcpy (value, pbuffer, (size_t) (p2buffer - pbuffer));
            value[p2buffer - pbuffer] = '\0';
        }

ERROR_LABEL:

    if (pbuffer != NULL)
        free (pbuffer);

    return rsrsc_status;
}

/*===========================================================================*/
/* Function: Set Quoted String                                               */
/* Purpose:  This function sets command with quoted string parameter.        */
/*===========================================================================*/
ViStatus rsrsc_setViStringQuoted (ViSession instrSession,
                            ViString cmd,
                            ViString quotes,
                            ViString value,
                            ViStatus parameterErrorValue)
{
    ViStatus    rsrsc_status = VI_SUCCESS;
    ViChar*     wrt_buf;
    ViChar*     p2buf;

    if (value == VI_NULL)
        return parameterErrorValue;

    wrt_buf = (ViChar*)malloc(strlen(cmd) + strlen(value) + 2*strlen(quotes) + 4);
    p2buf = wrt_buf;
    p2buf += sprintf(wrt_buf,"%s %s%s%s\n", cmd, quotes, value, quotes);

    GOCHECKERR (viWrite (instrSession, (ViBuf)wrt_buf, (ViUInt32) (p2buf - wrt_buf), VI_NULL));
    GOCHECKERR (rsrsc_checkStatus(instrSession));

ERROR_LABEL:

    free (wrt_buf);

    return rsrsc_status;
}

/*===========================================================================*/
/* Function: Set String With Attenuator                                      */
/* Purpose:  This function sets command with string parameter.               */
/*===========================================================================*/
ViStatus rsrsc_setViStringAtten (ViSession instrSession,
                            ViString cmd,
                            ViInt32 attenuator,
                            ViString value,
                            ViStatus parameterErrorValue)
{
    ViStatus    rsrsc_status = VI_SUCCESS;
    ViChar*     wrt_buf;
    ViChar*     p2buf;

    if (value == VI_NULL)
        return parameterErrorValue;

    wrt_buf = (ViChar*) malloc (strlen(cmd) + strlen(value) + 11);
    p2buf = wrt_buf;
    p2buf += sprintf(wrt_buf, cmd, attenuator);
    p2buf += sprintf(p2buf," %s\n", value);

    rsrsc_status = viWrite (instrSession, (ViBuf)wrt_buf, (ViUInt32)(p2buf - wrt_buf), VI_NULL);
    free(wrt_buf);

    CHECKERR(rsrsc_status);
    CHECKERR(rsrsc_checkStatus(instrSession));

    return rsrsc_status;
}

/*===========================================================================*/
/* Function: Get String with Attenuator                                      */
/* Purpose:  This function gets command with string parameter.               */
/*===========================================================================*/
ViStatus rsrsc_getViStringAtten (ViSession instrSession,
                                ViString cmd,
                                ViInt32 attenuator,
                                ViString value,
                                ViInt32 buffer_size,
                                ViStatus parameterErrorCode)
{
    ViChar      wrt_buf[BUFFER_SIZE];

    sprintf(wrt_buf,cmd, attenuator);

    return rsrsc_getViString(instrSession,
                             wrt_buf,
                             value,
                             (ViUInt32)buffer_size,
                             parameterErrorCode);
}


/*===========================================================================*/
/* Function: Set Enum With Attenuator                                        */
/* Purpose:  This function sets command with enumerated parameter.           */
/*===========================================================================*/
ViStatus rsrsc_setEnumAtten (ViSession instrSession,
                        ViString cmd,
                        ViInt32 attenuator,
                        ViInt32 value,
                        ViString list[],
                        ViStatus parameterErrorCode)
{
    ViStatus rsrsc_status = VI_SUCCESS;

    CHECKERR (rsrsc_checkIndexInList (value, list, parameterErrorCode));

    return rsrsc_setViStringAtten (instrSession, cmd, attenuator, list[value], parameterErrorCode);
}

/*===========================================================================*/
/* Function: Get Enum Attenuator                                             */
/* Purpose:  This function sets command with enumerated parameter.           */
/*===========================================================================*/
ViStatus rsrsc_getEnumAtten (ViSession instrSession,
                        ViString cmd,
                        ViInt32 attenuator,
                        ViInt32 *value,
                        ViString list[],
                        ViStatus parameterErrorCode)
{
    ViStatus    rsrsc_status = VI_SUCCESS;
    ViChar      buffer[BUFFER_SIZE] = "";

    CHECKERR(rsrsc_getViStringAtten(instrSession, cmd, attenuator, buffer, (ViInt32)BUFFER_SIZE, parameterErrorCode));
    CHECKERR( rsrsc_getIndexFromString (buffer, list, value));

    return rsrsc_status;

}



/*===========================================================================*/
/* Function: Check Index List                                                */
/* Purpose:  This function checks if the passed index is valid for given list*/
/*===========================================================================*/
ViStatus rsrsc_checkIndexInList (ViInt32 index,
                                 ViString list[],
                                 ViStatus statusIfError)

{
    ViInt32     i=0;

    if (index < 0)
        return statusIfError;

    while (list[i++] != VI_NULL);
    if (index >= i)
        return statusIfError;

    return VI_SUCCESS;
}

/*===========================================================================*/
/* Function: Get Index From String                                           */
/* Purpose:  This function returns index for string.                         */
/*===========================================================================*/
ViStatus rsrsc_getIndexFromString (ViString string,
                                   ViString *list,
                                   ViInt32 *index)
{
    ViStatus    rsrsc_status = VI_SUCCESS;
    ViUInt32    i;
    size_t      len;

    for (i = 0; list[i]; i++)
        if ((len = strlen (string)) == strlen (list[i])) {
            if (strncmp(string, list[i], strlen (string)) == 0) break;
        }

    *index = (ViInt32) i;

    return (list[i]) ? rsrsc_status : VI_ERROR_INSTR_INTERPRETING_RESPONSE;
}

/*===========================================================================*/
/* Function: Read Data With Unknown length                                   */
/* Purpose:  This function allocates buffer and reads data from the          */
/*           instrument                                                      */
/* WARNING:  You must deallocate buffer, if any pointer has been returned.   */
/*           This function does not work correctly, when you read bin data   */
/*           via serial intf                                                 */
/*===========================================================================*/
ViStatus rsrsc_readDataUnknownLength(ViSession instrSession,
                                     /*@null@*/ViChar **outBuffer,
                                     ViUInt32 *count)
{
    ViStatus rsrsc_status = VI_SUCCESS;
    ViUInt32 actSize, retCnt = 0;
    ViChar   *buffer = NULL, *p2buffer = NULL;

    *outBuffer = VI_NULL;
    if (count) *count = 0;

    /* Allocate Buffer */
    actSize = BUFFER_SIZE;
    buffer = (ViChar *)calloc((size_t)actSize, (size_t) sizeof(ViChar));
    if (buffer == VI_NULL) return VI_ERROR_ALLOC;

    p2buffer = buffer;
    do
    {
        if ((rsrsc_status = viRead (instrSession, (ViPBuf)p2buffer, BUFFER_SIZE, &retCnt)) < 0)
        {
            free(buffer);
            *outBuffer = VI_NULL;
            return rsrsc_status;
        }
        if (retCnt == BUFFER_SIZE) /* Buffer was small, reallocate it */
        {
            actSize += BUFFER_SIZE;
            buffer = (ViChar *)realloc(buffer, (size_t) actSize);
            if (buffer == VI_NULL) return VI_ERROR_ALLOC;
            p2buffer = buffer + (actSize - BUFFER_SIZE); /* Set pointer to end of data in rallocated buffer */
        }
        else
        {
            *(p2buffer+retCnt) = '\0'; /* For sure set end of string at the end of data */
        }
    } while (rsrsc_status == VI_SUCCESS_MAX_CNT); /* if buffer was small, continue with next read */

    *outBuffer = buffer;
    if (count != NULL) *count = (actSize - BUFFER_SIZE) + retCnt;

    return rsrsc_status;
}

